#! /usr/bin/perl
use strict;
use Data::Dumper;
use lib ( "." );
use QlogSplitter;

# ===========================================================================
# Splits a log file into multiple "tenants".
# Each tenant is rewritten as to appear an independent system.
# Requires qloader 1.29 or newer. 
#
# $Id: qlogSplitter.pl,v 1.1 2012/12/21 10:24:35 lenz-mobile Exp $
# ===========================================================================

syslog( "============= " . '$Id: qlogSplitter.pl,v 1.1 2012/12/21 10:24:35 lenz-mobile Exp $' . " ===========" );

# print Dumper( $CFG::SYS );

my $rowsProcessed = 0;
my $startedAt = 0;
my $lastInode = 0;


# stoppa i qloaders - tutti

my %systems = %{$CFG::SYS};

foreach  my $sys ( keys %systems ) {
	killQloader( $sys );
}

# crea i files di log
# troncati a zero

foreach  my $sys ( keys %systems ) {
	my $f = getFilename( $sys ); 
	syslog( "Truncating file $f");
	open F, ">$f" or die "$! $f";
	close F;
}


# legge il file sino alla fine
# effettuando lo split

my $file = $CFG::SRC;
open FX, $file or die "$! $file";
reopenFileIfRotated();

$startedAt = time();
while ( <FX> ) {
	my $row = $_;
	next if incompleteLine( $row );
	splitData( explodeRow( $row ) );
}



# restart dei qloader

foreach  my $sys ( keys %systems ) {
	startQloader( $sys );
}



# tail del file
# effettuando lo split

while ( 1 ) {
	while ( <FX> ) {
		my $row = $_;
		next if incompleteLine( $row );
		splitData( explodeRow( $row ) );
	}

	sleep 1;
	# Check for file rotation
	reopenFileIfRotated();
}


#
# cerca tutti i qloader lanciati da me - con l'opzione -sSYS_....
# e li chiude
#

sub killQloader {
	my ($sys) = @_;

	my @lines = `ps aux`;
	foreach my $l (@lines ) {
		if ( $l =~ /\s-sSYS_$sys\s/ ) {
			if ( $l =~ /^.+?\s+(\d+)\s+/ ) {
				syslog( "Stopping qloader for system $sys now running on PID $1" );				
				`kill -9 $1`;
			}
		}
	}
}

#
# fa partire ogni qloader con l'opzione -sSYS_... per poterli poi chiudere
#

sub startQloader {
	my ( $sys ) = @_;

	my %systems = %{$CFG::SYS};
	my %qldr = %{$systems{$sys}{'qloader'}};
	my $host = $qldr{'host'};
	my $db   = $qldr{'db'};
	my $user = $qldr{'user'};
	my $pass = $qldr{'pass'};
	my $part = $qldr{'partition'};
	my $qlog = getFilename($sys);

	my $qloader = $CFG::QLOADER_BIN;
	my $logsTo  = $CFG::QLOADER_LOG;
	$logsTo =~ s/#/$sys/;

	my $cmd = "$qloader -sSYS_$sys -h$host -d$db -u$user -p$pass $qlog $part $logsTo 0";

	syslog( "Running $cmd" );
	# lancia il comando
	system( "$cmd > /dev/null &" );	
}

sub explodeRow {
	my ($row) = @_;

	chomp $row;
	my @rowdata = ((split /\|/, $row), ( "", "", "", "", "", "", "", "", "")) ;
	$rowdata[0] = $rowdata[0] * 1.0;
	return @rowdata;
}

sub splitData {
	my ($tst, $cid, $que, $age, $verb, $d1, $d2, $d3, $d4, $d5) = @_;

	if ( $tst < 1000 ) {
		return;
	}

	my $rowLogged = 0;
	foreach  my $sys ( keys %systems ) {

		my $logInThisPartition = 0;

		if ( ( $que eq 'NONE') && ( $age eq 'NONE') ) {
			$logInThisPartition = 1;
		} else {

			my $regexp_q = $systems{$sys}{'match_q'};
			if ( $que =~ $regexp_q ) {
				$que = $1;
				$logInThisPartition = 1;
			} 

			my $regexp_a = $systems{$sys}{'match_a'};
			if ( $age =~ $regexp_q ) {
				$age = $1;
				$logInThisPartition = 1;
			} 

		}

		if ( $logInThisPartition ) {
			logToSystem( $sys, "$tst|$cid|$que|$age|$verb|$d1|$d2|$d3|$d4|$d5");
			$rowLogged += 1;
		}

	}

	if ( $rowLogged == 0 ) {
		logToSystem( "_UNKNOWN_", "$tst|$cid|$que|$age|$verb|$d1|$d2|$d3|$d4|$d5");	
	}


}

sub logToSystem {
	my ($system, $logline ) = @_;
	my $fn = getFilename( $system );
	open P, ">>$fn" or die "$! $fn";
	print P $logline . "\n";
	close P;

	$rowsProcessed += 1;

	if ( $rowsProcessed % 100 == 0 ) {
		my $elapsed = time() - $startedAt;
		my $rowsSec = 0;
		if ( $elapsed > 0 ) {
			$rowsSec = $rowsProcessed / $elapsed;
		}

		syslog( "Copied $rowsProcessed records - $rowsSec rows/s " );
	}


	#syslog( "$system -> $logline ");
}


#
# funzioni 
#

sub reopenFileIfRotated {    

	my $file = $CFG::SRC;

    my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks) = stat($file);
    if (($ino != "") && ($ino != $lastInode)) {
        close FX;
        syslog("Reopening file $file because it seems to have been rotated.");
        while (!open FX, $file) {
            syslog("File $file not found. Retrying in 10 seconds");
            sleep 10;
        }

        ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks) = stat($file);
        $lastInode=$ino;
    }

}

sub getFilename {
	my ($system) = @_;
	my $f = $CFG::QLOG;
	$f =~ s/#/$system/;
	return $f;
}

sub incompleteLine  {

	my ($read) = @_;

	# innanzi tutto, controllo di avere una riga completa
	# se non c'e', loop
	if ( index( $read, "\n") < 0 ) {

		# Check for file rotation
		#   $lastInode = checkForFileRotation($file, $lastInode);

		seek(FX, -length($_), 1);
		syslog( "Incomplete line read: pushing back '$_'" );
		sleep 1;		
		return 1;

	} else {
		return 0;
	}
}


# =================================================================
# 
#                      Error logging
#
# =================================================================


sub syslog {
	my ($s) = @_;
	syslogger( " ", $s );
}

sub syserr {
	my ($s) = @_;
	syslogger( "E", $s );
}


sub syslogger {
	my ($l, $s) = @_;
	$s =~ s/\n/ /gm;
	print "$l|$s\n";
	open EL, ">>$CFG::LOGFILE" or die "$! $CFG::LOGFILE";
	print EL "$l|" . (scalar localtime) . "|$s\n";
	close EL;
}

# ===================================================================
# $Log: qlogSplitter.pl,v $
# Revision 1.1  2012/12/21 10:24:35  lenz-mobile
# First public version
#
#
#




